clear 
close all
clc

%% ============= External dynamicis: Ship - FPSO collision ===============
% ========================================================================
% ------------------------------------------------------------------------
%           Stronge3d-function made by: Dr. Z. Liu, March, 2013
%           Input script by:            S.A. Hagen, April, 2018
% ------------------------------------------------------------------------

%   Object A: FPSO 
%   Object B: Shuttle tanker


%% ======================== Impact scenario ==============================
    angle = 30;                     % [deg.]    Impact angle
    Vel   = 1.9;                    % [m/s]     Impact velocity    
    
% ========================================================================
%                             FPSO data
% ========================================================================
% Type: Sevan 1000
Ha = 42.00;             % [m]   Height      (main deck)
Da = 93.0;              % [m]   Diameter    (Main hull - Ballast)
%Da = 104.0;            % [m]   Diameter    (Main deck - Full load)
Ta = 20.00;             % [m]   Draft       (Ballast WL)
%Ta = 28.30;            % [m]   Draft       (Full load WL)
Cxa = 0;                % [m]   COG x 
Cya = 0;                % [m]   COG y
Cza = 27.7;             % [m]   COG z       (Ballast, from bottom)
%Cza = 26.1;            % [m]   COG z       (Full load, from bottom)
Zga = Ta - Cza;         % [m]   Vertical distance from waterline to COG
%m_fpso = 215.0E6;      % [kg]  FPSO mass   (Fully loaded)
m_fpso = 155.0E6;       % [kg]  FPSO mass   (Ballast)
                                    
% Assumed added mass factors:
Amx = 1.0;                          %       Surge
Amy = 1.0;                          %       Sway
Amz = 1.0;                          %       Heave
Am = [Amx,Amy,Amz];
Amrol = 1.0;                        %       Roll
Ampit = 1.0;                        %       Pitch
Amyaw = 1.0;                        %       Yaw
Amr=[Amrol,Ampit,Amyaw];

% Inertia radius squared:       
Ixx = (1/12)*m_fpso*(3*(93/2)^2 + Ha^2);    % mass moment of inetria 
                                            % for cylinder (= Iyy). 
rxa = Ixx/m_fpso;       % [m^2]
rya = Ixx/m_fpso;       % [m^2]
rza = 0.5*(93/2)^2;     % [m^2]             Main hull diameter = 93.0 m
Ra  = [rxa,rya,rza];

% Writing out FPSO data:
fprintf('\n---------------- FPSO data --------------\n');
fprintf('Inertia radius squared: \n');
fprintf('\tRoll   [m^2]   = %f\n', rxa);
fprintf('\tPitch  [m^2]   = %f\n', rya);
fprintf('\tYaw    [m^2]   = %f\n', rza);
fprintf(' - - - - - - - - - - - - - - - - - - - - - - - -\n');
fprintf('\tRoll   [m]   = %f\n', sqrt(rxa));
fprintf('\tPitch  [m]   = %f\n', sqrt(rya));
fprintf('\tYaw    [m]   = %f\n', sqrt(rza));
fprintf(' - - - - - - - - - - - - - - - - - - - - - - - -\n');
    
% ======================================================================== 
%                           Shuttle tanker data
% ========================================================================
% Type: Samsung, Jasmine Knutsen
Hb  = 28.600;           % [m]   Height (from FEM model) 
Lb  = 262.00;           % [m]   Length
Bb  = 46.00;            % [m]   Breadth     
%Tb  = 9.3;             % [m]   Draft Ballast
Tb  = 19.0;             % [m]   Draft design (fully loaded)       
m_ship = 150E06;        % [kg]  Tanker mass (fully loaded)
%m_ship = 90E06;        % [kg]  Tanker mass (ballast)
Cxb = Lb/2;             % [m]   CoG x (assumed)
Cyb = 0;                % [m]   CoG y (assumed)
Czb = 12.650;           % [m]   CoG z (assumed), Ballast + Full load
                        %                     -> from bottom of ship
Zgb = Tb - Czb;         % [m]   Vertical distance from water line to CoG
Cwp = 0.9;              % [-]   Waterplane coefficent
Cm  = 0.9;              % [-]   Mid-section coefficient
Cb  = 0.8;              % [-]   Block cooeficent

% Added mass factors (purely empirical):
Bmx = 0.0;                          %       Surge
Bmy = 2*Tb/Bb;                      %       Sway
Bmz = 2/3*Bb*Cwp^2/(Tb*Cb*(1+Cwp)); %       Heave
Bm = [Bmx,Bmy,Bmz];
Bmrol = 0.25;                       %       Roll
Bmpit = Bb/(Tb*(3-2*Cwp)*(3-Cwp));  %       Pitch
Bmyaw = 0.3 + 0.05*(Lb/Bb);         %       Yaw
Bmr=[Bmrol,Bmpit,Bmyaw];

% Inertia radius squared:
rxb = (Cwp*Bb^2)/(11.4*Cm)+Hb^2/12; %       Rx^2
ryb = 0.07*Cwp*Lb^2;                %       Ry^2
rzb = Lb^2/16;                      %       Rz^2
Rb = [rxb,ryb,rzb];

fprintf('--------------- Tanker data ------------------\n');
fprintf('Added mass factor:\n');
fprintf('\tAmx    = %f\n', Bmx);
fprintf('\tAmy    = %f\n', Bmy);
fprintf('\tAmz    = %f\n', Bmz);
fprintf('\tRoll   = %f\n', Bmrol);
fprintf('\tPitch  = %f\n', Bmpit);
fprintf('\tYaw    = %f\n', Bmyaw);

% Writing out inertia radius for tanker:
fprintf('Inertia radius squared: \n');
fprintf('\tRoll   [m^2]     = %f\n', rxb);
fprintf('\tPitch  [m^2]     = %f\n', ryb);
fprintf('\tYaw    [m^2]     = %f\n', rzb);
fprintf(' - - - - - - - - - - - - - - - - - - - - - - - -\n');
fprintf('\tIxx    [tonn*mm^2] = %e\n', rxb*m_ship*1E3);
fprintf('\tIyy    [tonn*mm^2] = %e\n', ryb*m_ship*1E3);
fprintf('\tIzz    [tonn*mm^2] = %e\n', rzb*m_ship*1E3);
fprintf(' - - - - - - - - - - - - - - - - - - - - - - - -\n');
fprintf('Inertia radius: \n');
fprintf('\tRoll   [m]     = %f\n', sqrt(rxb));
fprintf('\tPitch  [m]     = %f\n', sqrt(ryb));
fprintf('\tYaw    [m]     = %f\n', sqrt(rzb));
fprintf(' - - - - - - - - - - - - - - - - - - - - - - - -\n');

%% ========================= Contact surface =============================
% FPSO in ballast: 
alpha = 90*cosd(angle); % [deg] Waterline angle
gama = 0;               % [deg] The angle between body frame A and B
betap = 0;              % [deg] The normal frame angle 
% FPSO in full load: 
%alpha = 90*cosd(angle);% [deg] Waterline angle
%gama = 0;              % [deg] The angle between body frame A and B
%betap = -26.6;         % [deg] The normal frame angle 

% Collision point under tanker's body frame: 
% Collision point under FPSO's body frame:% 
cp_a = [-(Da/2)*cosd(angle)                 % x-coordinate
        -(Da/2)*sind(angle)                 % y-coordinate
        -(Cza-3.25)];                       % z-coordinate (ballast)
                                            % Height BB: 3.25 m
% Assuming semicircle bow (in the horizontal plane)
cp_b = [(Lb/2 - (Bb/2)*(1 - cosd(angle)))   % x-coordinate
        (Bb/2)*sind(angle)                  % y-coordinate
        -Czb];                              % z-coordinate (fully loaded)

%% ========================= Impact velocity =============================
% Velocity of FPSO under FPSO's body frame:
ve_a = [0 0 0]';   %[m/s]    [Vx Vy Vz]

% Velocity of ship under ship's body frame:
ve_b = [Vel 0 0]'; %[m/s]    [Vx Vy Vz]

%% =========================== Friction ==================================
% Static friction between ship and FPSO:
%   Steel-to-steel factor = 0.3
%   tangential deformation factor (equivalent) = 0.3
%   => Efficient friction factor = 0.6
miu0 = 0.6;

% Restitution factor e (0 = plastic, 1 = elastic):
res = 0;

%% ========================= Run analysis ================================
% INFO: the "strong3d()" subroutine is available from the Department of 
% Marine Technology at Norwegian University of Science and Technology
% (NTNU)

[tt,ttm,dvv,ve_af,ve_bf,flag,miu,mass1,mass2] = ...
    stronge3d(m_fpso,m_ship,Am,Bm,Amr,Bmr,Ra,Rb,alpha,gama,...
    betap,cp_a,cp_b,res,miu0,ve_a,ve_b);

% OUTPUT PARAMETERS:
% tt    : total dissipated energy [J]
% ttm   : an array for dissipated energy in each direction [J]
% dvv   : relative velocity increase under the local frame n1n2n3
% ve_af : velocity after impact of object a under body frame of object a
% ve_bf : velocity after impact of object b under body frame of object b
% flag  : stick (1) or slide (2)
% miu   : static friction factor between object a and b
% mass1 : mass matrix for object a
% mass2 : mass matrix for object b

%% ============================ RESULTS ==================================
ve_af = double(ve_af);
ve_bf = double(ve_bf);
% ------------------- Write out results ------------------------
fprintf('\n\n----------------- RESULTS -----------------\n');
fprintf('Total dissipated energy:\n');
fprintf('\tE_tot [MJ]  = %6.2f\n', tt*1E-06);
fprintf('\nDissipated energy in each direction');
fprintf('(local coordinate system):\n'); 
fprintf('\tEx    [MJ]  = %6.2f\n', ttm(1)*1E-06);
fprintf('\tEy    [MJ]  = %6.2f\n', ttm(2)*1E-06);
fprintf('\tEz    [MJ]  = %6.2f\n\n', ttm(3)*1E-06);
fprintf('\tEr    [MJ]  = %6.2f   (Parallel to impact surface)\n', ...
    (ttm(1) + ttm(2))*1E-06);
fprintf('\tEz    [MJ]  = %6.2f   (Perpendicular to impact surface)\n', ...
    ttm(3)*1E-06);
fprintf('\nFPSO velocity after impact:\n');
fprintf('\tVx    [m/s] = %f\n', ve_af(1));
fprintf('\tVy    [m/s] = %f\n', ve_af(2));
fprintf('\tVz    [m/s] = %f\n', ve_af(3));
fprintf('\nShip velocity after impact:\n');
fprintf('\tVx    [m/s] = %f\n', ve_bf(1));
fprintf('\tVy    [m/s] = %f\n', ve_bf(2));
fprintf('\tVz    [m/s] = %f\n', ve_bf(3));

fprintf('\nImpact type:\n'); 
fprintf('\tCase: %s\n', flag);
fprintf('\tNormal friction coeffisient:');
fprintf('%f (static friction factor?)\n', miu); 

% END OF INPUT-SCRIPT
